@@ -32,6 +32,9 @@ CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 #include "events.h"
 #include "thumbnail.h"
 
+#define FEH_JITTER_OFFSET 2
+#define FEH_JITTER_TIME 1
+
 fehbb buttons;
 
 feh_event_handler *ev_handler[LASTEvent];
@@ -149,6 +152,10 @@ void init_buttonbindings(void)
 			cur_bb = &buttons.blur;
 		else if (!strcmp(action, "rotate"))
 			cur_bb = &buttons.rotate;
+		else if (!strcmp(action, "zoom_in"))
+			cur_bb = &buttons.zoom_in;
+		else if (!strcmp(action, "zoom_out"))
+			cur_bb = &buttons.zoom_out;
 		else
 			weprintf("buttons: Invalid action: %s", action);
 
@@ -226,6 +233,7 @@ static void feh_event_handle_ButtonPress(XEvent * ev)
 		D(("click offset is %d,%d\n", ev->xbutton.x, ev->xbutton.y));
 		winwid->click_offset_x = ev->xbutton.x - winwid->im_x;
 		winwid->click_offset_y = ev->xbutton.y - winwid->im_y;
+		winwid->click_start_time = time(NULL);
 
 	} else if (feh_is_bb(&buttons.zoom, button, state)) {
 		D(("Zoom Button Press event\n"));
@@ -242,6 +250,62 @@ static void feh_event_handle_ButtonPress(XEvent * ev)
 		winwid->im_click_offset_y = (winwid->click_offset_y
 				- winwid->im_y) / winwid->old_zoom;
 
+	} else if (feh_is_bb(&buttons.zoom_in, button, state)) {
+		D(("Zoom_In Button Press event\n"));
+		D(("click offset is %d,%d\n", ev->xbutton.x, ev->xbutton.y));
+		winwid->click_offset_x = ev->xbutton.x;
+		winwid->click_offset_y = ev->xbutton.y;
+		winwid->old_zoom = winwid->zoom;
+
+		/* required to adjust the image position in zoom mode */
+		winwid->im_click_offset_x = (winwid->click_offset_x
+				- winwid->im_x) / winwid->old_zoom;
+		winwid->im_click_offset_y = (winwid->click_offset_y
+				- winwid->im_y) / winwid->old_zoom;
+
+		/* copied from zoom_in, keyevents.c */
+		winwid->zoom = winwid->zoom * 1.25;
+
+		if (winwid->zoom > ZOOM_MAX)
+			winwid->zoom = ZOOM_MAX;
+
+		/* copied from below (ZOOM, feh_event_handle_MotionNotify) */
+		winwid->im_x = winwid->click_offset_x
+				- (winwid->im_click_offset_x * winwid->zoom);
+		winwid->im_y = winwid->click_offset_y
+				- (winwid->im_click_offset_y * winwid->zoom);
+
+		winwidget_sanitise_offsets(winwid);
+		winwidget_render_image(winwid, 0, 0);
+
+	} else if (feh_is_bb(&buttons.zoom_out, button, state)) {
+		D(("Zoom_Out Button Press event\n"));
+		D(("click offset is %d,%d\n", ev->xbutton.x, ev->xbutton.y));
+		winwid->click_offset_x = ev->xbutton.x;
+		winwid->click_offset_y = ev->xbutton.y;
+		winwid->old_zoom = winwid->zoom;
+
+		/* required to adjust the image position in zoom mode */
+		winwid->im_click_offset_x = (winwid->click_offset_x
+				- winwid->im_x) / winwid->old_zoom;
+		winwid->im_click_offset_y = (winwid->click_offset_y
+				- winwid->im_y) / winwid->old_zoom;
+
+		/* copied from zoom_out, keyevents.c */
+		winwid->zoom = winwid->zoom * 0.80;
+
+		if (winwid->zoom < ZOOM_MIN)
+			winwid->zoom = ZOOM_MIN;
+
+		/* copied from below (ZOOM, feh_event_handle_MotionNotify) */
+		winwid->im_x = winwid->click_offset_x
+				- (winwid->im_click_offset_x * winwid->zoom);
+		winwid->im_y = winwid->click_offset_y
+				- (winwid->im_click_offset_y * winwid->zoom);
+
+		winwidget_sanitise_offsets(winwid);
+		winwidget_render_image(winwid, 0, 0);
+
 	} else if (feh_is_bb(&buttons.reload, button, state)) {
 		D(("Reload Button Press event\n"));
 			feh_reload_image(winwid, 0, 1);
@@ -496,8 +560,14 @@ static void feh_event_handle_MotionNotify(XEvent * ev)
 		winwid = winwidget_get_from_window(ev->xmotion.window);
 		if (winwid) {
 			if (opt.mode == MODE_NEXT) {
-				opt.mode = MODE_PAN;
-				winwid->mode = MODE_PAN;
+				if ((abs(winwid->click_offset_x - (ev->xmotion.x - winwid->im_x)) > FEH_JITTER_OFFSET)
+						|| (abs(winwid->click_offset_y - (ev->xmotion.y - winwid->im_y)) > FEH_JITTER_OFFSET)
+						|| (time(NULL) - winwid->click_start_time > FEH_JITTER_TIME)) {
+					opt.mode = MODE_PAN;
+					winwid->mode = MODE_PAN;
+				}
+				else
+					return;
 			}
 			D(("Panning\n"));
 			orig_x = winwid->im_x;
