@@ -59,17 +59,41 @@ int xinerama_screen;
 int num_xinerama_screens;
 #endif				/* HAVE_LIBXINERAMA */
 
+int childpid = 0;
+
+static char *feh_http_load_image(char *url);
+static char *feh_magick_load_image(char *filename);
+
 #ifdef HAVE_LIBXINERAMA
 void init_xinerama(void)
 {
 	if (opt.xinerama && XineramaIsActive(disp)) {
-		int major, minor;
-		if (getenv("XINERAMA_SCREEN"))
-			xinerama_screen = atoi(getenv("XINERAMA_SCREEN"));
-		else
-			xinerama_screen = 0;
+		int major, minor, px, py, i;
+
+		/* discarded */
+		Window dw;
+		int di;
+		unsigned int du;
+
 		XineramaQueryVersion(disp, &major, &minor);
 		xinerama_screens = XineramaQueryScreens(disp, &num_xinerama_screens);
+
+		if (getenv("XINERAMA_SCREEN"))
+			xinerama_screen = atoi(getenv("XINERAMA_SCREEN"));
+		else {
+			xinerama_screen = 0;
+			XQueryPointer(disp, root, &dw, &dw, &px, &py, &di, &di, &du);
+			for (i = 0; i < num_xinerama_screens; i++) {
+				if (XY_IN_RECT(px, py,
+							xinerama_screens[i].x_org,
+							xinerama_screens[i].y_org,
+							xinerama_screens[i].width,
+							xinerama_screens[i].height)) {
+					xinerama_screen = i;
+					break;
+				}
+			}
+		}
 	}
 }
 #endif				/* HAVE_LIBXINERAMA */
@@ -124,51 +148,132 @@ int feh_load_image_char(Imlib_Image * im, char *filename)
 	return(i);
 }
 
+/*
+ * XXX gib_imlib_save_image_with_error_return breaks with *.END and
+ * similar because it tries to set the image format, which only works
+ * with .end .
+ * So we leave that part out.
+ */
+void ungib_imlib_save_image_with_error_return(Imlib_Image im, char *file,
+	Imlib_Load_Error * error_return)
+{
+	char *tmp;
+	imlib_context_set_image(im);
+	tmp = strrchr(file, '.');
+	if (tmp) {
+		char *p, *pp;
+		p = gib_estrdup(tmp + 1);
+		pp = p;
+		while(*pp) {
+			*pp = tolower(*pp);
+			pp++;
+		}
+		imlib_image_set_format(p);
+		gib_efree(p);
+	}
+	imlib_save_image_with_error_return(file, error_return);
+}
+
+void feh_imlib_print_load_error(char *file, winwidget w, Imlib_Load_Error err)
+{
+	if (err == IMLIB_LOAD_ERROR_OUT_OF_FILE_DESCRIPTORS)
+		eprintf("%s - Out of file descriptors while loading", file);
+	else if (!opt.quiet || w) {
+		switch (err) {
+		case IMLIB_LOAD_ERROR_FILE_DOES_NOT_EXIST:
+			im_weprintf(w, "%s - File does not exist", file);
+			break;
+		case IMLIB_LOAD_ERROR_FILE_IS_DIRECTORY:
+			im_weprintf(w, "%s - Directory specified for image filename", file);
+			break;
+		case IMLIB_LOAD_ERROR_PERMISSION_DENIED_TO_READ:
+			im_weprintf(w, "%s - No read access", file);
+			break;
+		case IMLIB_LOAD_ERROR_UNKNOWN:
+		case IMLIB_LOAD_ERROR_NO_LOADER_FOR_FILE_FORMAT:
+			im_weprintf(w, "%s - No Imlib2 loader for that file format", file);
+			break;
+		case IMLIB_LOAD_ERROR_PATH_TOO_LONG:
+			im_weprintf(w, "%s - Path specified is too long", file);
+			break;
+		case IMLIB_LOAD_ERROR_PATH_COMPONENT_NON_EXISTANT:
+			im_weprintf(w, "%s - Path component does not exist", file);
+			break;
+		case IMLIB_LOAD_ERROR_PATH_COMPONENT_NOT_DIRECTORY:
+			im_weprintf(w, "%s - Path component is not a directory", file);
+			break;
+		case IMLIB_LOAD_ERROR_PATH_POINTS_OUTSIDE_ADDRESS_SPACE:
+			im_weprintf(w, "%s - Path points outside address space", file);
+			break;
+		case IMLIB_LOAD_ERROR_TOO_MANY_SYMBOLIC_LINKS:
+			im_weprintf(w, "%s - Too many levels of symbolic links", file);
+			break;
+		case IMLIB_LOAD_ERROR_OUT_OF_MEMORY:
+			im_weprintf(w, "While loading %s - Out of memory", file);
+			break;
+		case IMLIB_LOAD_ERROR_PERMISSION_DENIED_TO_WRITE:
+			im_weprintf(w, "%s - Cannot write to directory", file);
+			break;
+		case IMLIB_LOAD_ERROR_OUT_OF_DISK_SPACE:
+			im_weprintf(w, "%s - Cannot write - out of disk space", file);
+			break;
+		default:
+			im_weprintf(w, "While loading %s - Unknown error (%d)",
+					file, err);
+			break;
+		}
+	}
+}
+
 int feh_load_image(Imlib_Image * im, feh_file * file)
 {
-	Imlib_Load_Error err;
+	Imlib_Load_Error err = IMLIB_LOAD_ERROR_NONE;
+	enum { SRC_IMLIB, SRC_HTTP, SRC_MAGICK } image_source =
+		SRC_IMLIB;
+	char *tmpname = NULL;
+	char *real_filename = NULL;
+
+#ifdef HAVE_LIBEXIF
+	ExifEntry *entry;
+#endif
 
 	D(("filename is %s, image is %p\n", file->filename, im));
 
 	if (!file || !file->filename)
-		return(0);
+		return 0;
 
 	/* Handle URLs */
 	if ((!strncmp(file->filename, "http://", 7)) || (!strncmp(file->filename, "https://", 8))
 			|| (!strncmp(file->filename, "ftp://", 6))) {
-		char *tmpname = NULL;
-		char *tempcpy;
+		image_source = SRC_HTTP;
 
-		tmpname = feh_http_load_image(file->filename);
-		if (tmpname == NULL)
-			return(0);
+		if ((tmpname = feh_http_load_image(file->filename)) == NULL)
+			err = IMLIB_LOAD_ERROR_FILE_DOES_NOT_EXIST;
+	}
+	else
+		*im = imlib_load_image_with_error_return(file->filename, &err);
+
+	if ((err == IMLIB_LOAD_ERROR_UNKNOWN)
+			|| (err == IMLIB_LOAD_ERROR_NO_LOADER_FOR_FILE_FORMAT)) {
+		image_source = SRC_MAGICK;
+		tmpname = feh_magick_load_image(file->filename);
+	}
+
+	if ((image_source != SRC_IMLIB) && tmpname) {
 		*im = imlib_load_image_with_error_return(tmpname, &err);
 		if (im) {
-			/* load the info now, in case it's needed after we delete the
-			   temporary image file */
-			tempcpy = file->filename;
+			real_filename = file->filename;
 			file->filename = tmpname;
 			feh_file_info_load(file, *im);
-			file->filename = tempcpy;
+			file->filename = real_filename;
 #ifdef HAVE_LIBEXIF
 			file->ed = exif_get_data(tmpname);
-#endif		
+#endif
 		}
-		if ((opt.slideshow) && (opt.reload == 0)) {
-			/* Http, no reload, slideshow. Let's keep this image on hand... */
-			free(file->filename);
-			file->filename = estrdup(tmpname);
+		if ((image_source == SRC_MAGICK) || !opt.keep_http)
+			unlink(tmpname);
 
-			if (!opt.keep_http)
-				add_file_to_rm_filelist(tmpname);
-		} else {
-			/* Don't cache the image if we're doing reload + http (webcams etc) */
-			if (!opt.keep_http)
-				unlink(tmpname);
-		}
 		free(tmpname);
-	} else {
-		*im = imlib_load_image_with_error_return(file->filename, &err);
 	}
 
 	if ((err) || (!im)) {
@@ -176,68 +281,117 @@ int feh_load_image(Imlib_Image * im, feh_file * file)
 			fputs("\n", stdout);
 			reset_output = 1;
 		}
-		if (err == IMLIB_LOAD_ERROR_OUT_OF_FILE_DESCRIPTORS)
-			eprintf("While loading %s - Out of file descriptors", file->filename);
-		else if (!opt.quiet) {
-			switch (err) {
-			case IMLIB_LOAD_ERROR_FILE_DOES_NOT_EXIST:
-				weprintf("%s - File does not exist", file->filename);
-				break;
-			case IMLIB_LOAD_ERROR_FILE_IS_DIRECTORY:
-				weprintf("%s - Directory specified for image filename", file->filename);
-				break;
-			case IMLIB_LOAD_ERROR_PERMISSION_DENIED_TO_READ:
-				weprintf("%s - No read access to directory", file->filename);
-				break;
-			case IMLIB_LOAD_ERROR_UNKNOWN:
-			case IMLIB_LOAD_ERROR_NO_LOADER_FOR_FILE_FORMAT:
-				weprintf("%s - No Imlib2 loader for that file format", file->filename);
-				break;
-			case IMLIB_LOAD_ERROR_PATH_TOO_LONG:
-				weprintf("%s - Path specified is too long", file->filename);
-				break;
-			case IMLIB_LOAD_ERROR_PATH_COMPONENT_NON_EXISTANT:
-				weprintf("%s - Path component does not exist", file->filename);
-				break;
-			case IMLIB_LOAD_ERROR_PATH_COMPONENT_NOT_DIRECTORY:
-				weprintf("%s - Path component is not a directory", file->filename);
-				break;
-			case IMLIB_LOAD_ERROR_PATH_POINTS_OUTSIDE_ADDRESS_SPACE:
-				weprintf("%s - Path points outside address space", file->filename);
-				break;
-			case IMLIB_LOAD_ERROR_TOO_MANY_SYMBOLIC_LINKS:
-				weprintf("%s - Too many levels of symbolic links", file->filename);
-				break;
-			case IMLIB_LOAD_ERROR_OUT_OF_MEMORY:
-				weprintf("While loading %s - Out of memory", file->filename);
-				break;
-			case IMLIB_LOAD_ERROR_PERMISSION_DENIED_TO_WRITE:
-				weprintf("%s - Cannot write to directory", file->filename);
-				break;
-			case IMLIB_LOAD_ERROR_OUT_OF_DISK_SPACE:
-				weprintf("%s - Cannot write - out of disk space", file->filename);
-				break;
-			default:
-				weprintf("While loading %s - Unknown error (%d)",
-						file->filename, err);
-				break;
-			}
-		}
+		feh_imlib_print_load_error(file->filename, NULL, err);
 		D(("Load *failed*\n"));
 		return(0);
 	}
 
 #ifdef HAVE_LIBEXIF
 	file->ed = exif_get_data(file->filename);
-#endif		
+
+	if (file->ed) {
+		entry = exif_content_get_entry(file->ed->ifd[EXIF_IFD_0], 0x0112);
+		if (entry != NULL) {
+			if (*(entry->data) == 3)
+				gib_imlib_image_orientate(*im, 2);
+			else if (*(entry->data) == 6)
+				gib_imlib_image_orientate(*im, 1);
+			else if (*(entry->data) == 8)
+				gib_imlib_image_orientate(*im, 3);
+		}
+	}
+#endif
 
 	D(("Loaded ok\n"));
 	return(1);
 }
 
+static char *feh_magick_load_image(char *filename)
+{
+	char argv_fd[12];
+	char *basename;
+	char *tmpname;
+	char *sfn;
+	int fd = -1, devnull = -1;
+	int status;
+
+	if (opt.magick_timeout < 0)
+		return NULL;
+
+	basename = strrchr(filename, '/');
+
+	if (basename == NULL)
+		basename = filename;
+	else
+		basename++;
+
+	tmpname = feh_unique_filename("/tmp/", basename);
+
+	if (strlen(tmpname) > (NAME_MAX-6))
+		tmpname[NAME_MAX-7] = '\0';
+
+	sfn = estrjoin("_", tmpname, "XXXXXX", NULL);
+	free(tmpname);
+
+	fd = mkstemp(sfn);
+
+	if (fd == -1)
+		return NULL;
+
+	snprintf(argv_fd, sizeof(argv_fd), "png:fd:%d", fd);
+
+	if ((childpid = fork()) == 0) {
+
+		/* discard convert output */
+		devnull = open("/dev/null", O_WRONLY);
+		dup2(devnull, 0);
+		dup2(devnull, 1);
+		dup2(devnull, 2);
+
+		/*
+		 * convert only accepts SIGINT via killpg, a normal kill doesn't work
+		 */
+		setpgid(0, 0);
+
+		execlp("convert", "convert", filename, argv_fd, NULL);
+		exit(1);
+	}
+	else {
+		alarm(opt.magick_timeout);
+		waitpid(childpid, &status, 0);
+		alarm(0);
+		if (!WIFEXITED(status) || (WEXITSTATUS(status) != 0)) {
+			close(fd);
+			unlink(sfn);
+			free(sfn);
+			sfn = NULL;
+
+			if (!opt.quiet) {
+				if (WIFSIGNALED(status))
+					weprintf("%s - Conversion took too long, skipping",
+						filename);
+			}
+
+			/*
+			 * Reap child.  The previous waitpid call was interrupted by
+			 * alarm, but convert doesn't terminate immediately.
+			 * XXX
+			 * normally, if (WIFSIGNALED(status)) waitpid(childpid, &status, 0);
+			 * would suffice. However, as soon as feh has its own window,
+			 * this doesn't work anymore and the following workaround is
+			 * required. Hm.
+			 */
+			waitpid(-1, &status, 0);
+		}
+		childpid = 0;
+	}
+
+	return sfn;
+}
+
 #ifdef HAVE_LIBCURL
 
-char *feh_http_load_image(char *url)
+static char *feh_http_load_image(char *url)
 {
 	CURL *curl;
 	CURLcode res;
@@ -257,15 +411,15 @@ char *feh_http_load_image(char *url)
 	} else
 		path = "/tmp/";
 
-	basename = strrchr(url, '/') + 1;
-	tmpname = feh_unique_filename(path, basename);
-
 	curl = curl_easy_init();
 	if (!curl) {
 		weprintf("open url: libcurl initialization failure");
 		return NULL;
 	}
 
+	basename = strrchr(url, '/') + 1;
+	tmpname = feh_unique_filename(path, basename);
+
 	if (strlen(tmpname) > (NAME_MAX-6))
 		tmpname[NAME_MAX-7] = '\0';
 	
@@ -514,7 +668,7 @@ void feh_draw_exif(winwidget w)
 	int pos2 = 0;
 	char info_line[256];
 	char *info_buf[128];
-	char buffer[MAX_EXIF_DATA];
+	char buffer[EXIF_MAX_DATA];
 
 	if ( (!w->file) || (!FEH_FILE(w->file->data))
 			 || (!FEH_FILE(w->file->data)->filename) )
@@ -522,20 +676,22 @@ void feh_draw_exif(winwidget w)
 		return;
 	}
 
-	exif_get_info(FEH_FILE(w->file->data)->ed, buffer, MAX_EXIF_DATA);
+
+	buffer[0] = '\0';
+	exif_get_info(FEH_FILE(w->file->data)->ed, buffer, EXIF_MAX_DATA);
 
 	fn = feh_load_font(w);
 
 	if (buffer == NULL) 
 	{
-		snprintf(buffer, MAX_EXIF_DATA, "%s", estrdup("Failed to run exif command"));
+		snprintf(buffer, EXIF_MAX_DATA, "%s", estrdup("Failed to run exif command"));
 		gib_imlib_get_text_size(fn, &buffer[0], NULL, &width, &height, IMLIB_TEXT_TO_RIGHT);
 		no_lines = 1;
 	}
 	else 
 	{
 
-		while ( (no_lines < 128) && (pos < MAX_EXIF_DATA) )
+		while ( (no_lines < 128) && (pos < EXIF_MAX_DATA) )
 		{
 			/* max 128 lines */
 			pos2 = 0;
@@ -548,7 +704,7 @@ void feh_draw_exif(winwidget w)
 			  }
 			  else if ( buffer[pos] == '\0' )
 			  {
-			    pos = MAX_EXIF_DATA; /* all data seen */
+			    pos = EXIF_MAX_DATA; /* all data seen */
 			    info_line[pos2] = '\0';
 				}
 			  else
@@ -624,7 +780,7 @@ void feh_draw_info(winwidget w)
 
 	fn = feh_load_font(w);
 
-	info_cmd = feh_printf(opt.info_cmd, FEH_FILE(w->file->data));
+	info_cmd = feh_printf(opt.info_cmd, FEH_FILE(w->file->data), w);
 
 	info_pipe = popen(info_cmd, "r");
 
@@ -880,6 +1036,7 @@ void feh_edit_inplace(winwidget w, int op)
 {
 	int ret;
 	Imlib_Image old;
+	Imlib_Load_Error err;
 	if (!w->file || !w->file->data || !FEH_FILE(w->file->data)->filename)
 		return;
 
@@ -899,8 +1056,12 @@ void feh_edit_inplace(winwidget w, int op)
 			imlib_image_flip_horizontal();
 		} else
 			gib_imlib_image_orientate(old, op);
-		gib_imlib_save_image(old, FEH_FILE(w->file->data)->filename);
+		ungib_imlib_save_image_with_error_return(old,
+			FEH_FILE(w->file->data)->filename, &err);
 		gib_imlib_free_image(old);
+		if (err)
+			feh_imlib_print_load_error(FEH_FILE(w->file->data)->filename,
+				w, err);
 		feh_reload_image(w, 1, 1);
 	} else {
 		im_weprintf(w, "failed to load image from disk to edit it in place");
@@ -1013,6 +1174,7 @@ void feh_edit_inplace_lossless(winwidget w, int op)
 	int len = strlen(filename) + 1;
 	char *file_str = emalloc(len);
 	int pid, status;
+	int devnull = -1;
 	char op_name[]  = "rotate";     /* message */
 	char op_op[]    = "-rotate";    /* jpegtran option */
 	char op_value[] = "horizontal"; /* jpegtran option's value */
@@ -1031,15 +1193,17 @@ void feh_edit_inplace_lossless(winwidget w, int op)
 
 	if ((pid = fork()) < 0) {
 		im_weprintf(w, "lossless %s: fork failed:", op_name);
-		return;
-	} else if (pid == 0) {
+		exit(1);
+	}
+	else if (pid == 0) {
 
 		execlp("jpegtran", "jpegtran", "-copy", "all", op_op, op_value,
 				"-outfile", file_str, file_str, NULL);
 
 		im_weprintf(w, "lossless %s: Is 'jpegtran' installed? Failed to exec:", op_name);
-		return;
-	} else {
+		exit(1);
+	}
+	else {
 		waitpid(pid, &status, 0);
 
 		if (!WIFEXITED(status) || WEXITSTATUS(status) != 0) {
@@ -1048,9 +1212,35 @@ void feh_edit_inplace_lossless(winwidget w, int op)
 					" Commandline was: "
 					"jpegtran -copy all %s %s -outfile %s %s",
 					op_name, status >> 8, op_op, op_value, file_str, file_str);
+			free(file_str);
 			return;
 		}
 	}
+	if ((pid = fork()) < 0) {
+		im_weprintf(w, "lossless %s: cannot fix rotation: fork:", op_name);
+		exit(1);
+	}
+	else if (pid == 0) {
+
+		/* discard normal output */
+		devnull = open("/dev/null", O_WRONLY);
+		dup2(devnull, 1);
+
+		execlp("jpegexiforient", "jpegexiforient", "-1", file_str, NULL);
+		im_weprintf(w, "lossless %s: Failed to exec jpegexiforient:", op_name);
+		exit(1);
+	}
+	else {
+		waitpid(pid, &status, 0);
+
+		if (!WIFEXITED(status) || WEXITSTATUS(status) != 0) {
+			im_weprintf(w,
+					"lossless %s: Failed to update EXIF orientation tag:"
+					" jpegexiforient returned %d",
+					op_name, status >> 8);
+		}
+	}
+	free(file_str);
 }
 
 void feh_draw_actions(winwidget w)
