@@ -129,10 +129,10 @@ void init_keyevents(void) {
 	feh_set_kb(&keys.action_9  , 0, XK_9         , 0, XK_KP_9      , 0, 0);
 	feh_set_kb(&keys.zoom_in   , 0, XK_Up        , 0, XK_KP_Add    , 0, 0);
 	feh_set_kb(&keys.zoom_out  , 0, XK_Down      , 0, XK_KP_Subtract,0, 0);
-	feh_set_kb(&keys.zoom_default, 0, XK_KP_Multiply, 0, 0         , 0, 0);
-	feh_set_kb(&keys.zoom_fit  , 0, XK_KP_Divide , 0, 0            , 0, 0);
+	feh_set_kb(&keys.zoom_default, 0, XK_KP_Multiply, 0, XK_asterisk, 0, 0);
+	feh_set_kb(&keys.zoom_fit  , 0, XK_KP_Divide , 0, XK_slash     , 0, 0);
 	feh_set_kb(&keys.size_to_image, 0, XK_w      , 0, 0            , 0, 0);
-	feh_set_kb(&keys.render    , 0, XK_KP_Begin  , 0, 0            , 0, 0);
+	feh_set_kb(&keys.render    , 0, XK_KP_Begin  , 0, XK_R         , 0, 0);
 	feh_set_kb(&keys.toggle_actions, 0, XK_a, 0, 0, 0, 0);
 	feh_set_kb(&keys.toggle_aliasing, 0, XK_A, 0, 0, 0, 0);
 	feh_set_kb(&keys.toggle_filenames, 0, XK_d, 0, 0, 0, 0);
@@ -154,6 +154,7 @@ void init_keyevents(void) {
 	feh_set_kb(&keys.mirror, 0, XK_bar, 0, 0, 0, 0);
 	feh_set_kb(&keys.reload_minus, 0, XK_minus, 0, 0, 0, 0);
 	feh_set_kb(&keys.reload_plus, 0, XK_plus, 0, 0, 0, 0);
+	feh_set_kb(&keys.toggle_keep_vp, 0, XK_k, 0, 0, 0, 0);
 
 	home = getenv("HOME");
 	if (!home)
@@ -307,6 +308,8 @@ void init_keyevents(void) {
 			cur_kb = &keys.reload_minus;
 		else if (!strcmp(action, "reload_plus"))
 			cur_kb = &keys.reload_plus;
+		else if (!strcmp(action, "toggle_keep_vp"))
+			cur_kb = &keys.toggle_keep_vp;
 		else
 			weprintf("keys: Invalid action: %s", action);
 
@@ -479,34 +482,42 @@ void feh_event_handle_keypress(XEvent * ev)
 	}
 	else if (feh_is_kp(&keys.scroll_right, keysym, state)) {
 		winwid->im_x -= 20;
+		winwidget_sanitise_offsets(winwid);
 		winwidget_render_image(winwid, 0, 1);
 	}
 	else if (feh_is_kp(&keys.scroll_left, keysym, state)) {
 		winwid->im_x += 20;
+		winwidget_sanitise_offsets(winwid);
 		winwidget_render_image(winwid, 0, 1);
 	}
 	else if (feh_is_kp(&keys.scroll_down, keysym, state)) {
 		winwid->im_y -= 20;
+		winwidget_sanitise_offsets(winwid);
 		winwidget_render_image(winwid, 0, 1);
 	}
 	else if (feh_is_kp(&keys.scroll_up, keysym, state)) {
 		winwid->im_y += 20;
+		winwidget_sanitise_offsets(winwid);
 		winwidget_render_image(winwid, 0, 1);
 	}
 	else if (feh_is_kp(&keys.scroll_right_page, keysym, state)) {
 		winwid->im_x -= winwid->w;
+		winwidget_sanitise_offsets(winwid);
 		winwidget_render_image(winwid, 0, 0);
 	}
 	else if (feh_is_kp(&keys.scroll_left_page, keysym, state)) {
 		winwid->im_x += winwid->w;
+		winwidget_sanitise_offsets(winwid);
 		winwidget_render_image(winwid, 0, 0);
 	}
 	else if (feh_is_kp(&keys.scroll_down_page, keysym, state)) {
 		winwid->im_y -= winwid->h;
+		winwidget_sanitise_offsets(winwid);
 		winwidget_render_image(winwid, 0, 0);
 	}
 	else if (feh_is_kp(&keys.scroll_up_page, keysym, state)) {
 		winwid->im_y += winwid->h;
+		winwidget_sanitise_offsets(winwid);
 		winwidget_render_image(winwid, 0, 0);
 	}
 	else if (feh_is_kp(&keys.jump_back, keysym, state)) {
@@ -650,7 +661,10 @@ void feh_event_handle_keypress(XEvent * ev)
 		opt.hide_pointer = !opt.hide_pointer;
 	}
 	else if (feh_is_kp(&keys.jump_random, keysym, state)) {
-		slideshow_change_image(winwid, SLIDE_RAND, 1);
+		if (winwid->type == WIN_TYPE_THUMBNAIL)
+			feh_thumbnail_select_next(winwid, rand() % (filelist_len - 1));
+		else
+			slideshow_change_image(winwid, SLIDE_RAND, 1);
 	}
 	else if (feh_is_kp(&keys.toggle_caption, keysym, state)) {
 		if (opt.caption_path) {
@@ -742,7 +756,7 @@ void feh_event_handle_keypress(XEvent * ev)
 		if (opt.reload < SLIDESHOW_RELOAD_MAX)
 			opt.reload++;
 		else if (opt.verbose)
-			weprintf("Cannot set RELOAD higher than %d seconds.", opt.reload);
+			weprintf("Cannot set RELOAD higher than %f seconds.", opt.reload);
 	}
 	else if (feh_is_kp(&keys.reload_minus, keysym, state)) {
 		if (opt.reload > 1)
@@ -750,5 +764,8 @@ void feh_event_handle_keypress(XEvent * ev)
 		else if (opt.verbose)
 			weprintf("Cannot set RELOAD lower than 1 second.");
 	}
+	else if (feh_is_kp(&keys.toggle_keep_vp, keysym, state)) {
+		opt.keep_zoom_vp = !opt.keep_zoom_vp;
+	}
 	return;
 }
