@@ -37,19 +37,22 @@ void init_list_mode(void)
 	mode = "list";
 
 	if (!opt.customlist)
-		fputs("NUM\tFORMAT\tWIDTH\tHEIGHT\tPIXELS\tSIZE(bytes)\tALPHA\tFILENAME\n",
+		fputs("NUM\tFORMAT\tWIDTH\tHEIGHT\tPIXELS\tSIZE\tALPHA\tFILENAME\n",
 				stdout);
 
 	for (l = filelist; l; l = l->next) {
 		file = FEH_FILE(l->data);
 		if (opt.customlist)
-			printf("%s\n", feh_printf(opt.customlist, file));
-		else
-			printf("%d\t%s\t%d\t%d\t%d\t%d\t\t%c\t%s\n", ++j,
+			printf("%s\n", feh_printf(opt.customlist, file, NULL));
+		else {
+			printf("%d\t%s\t%d\t%d\t%s", ++j,
 					file->info->format, file->info->width,
-					file->info->height, file->info->pixels,
-					file->info->size,
+					file->info->height,
+					format_size(file->info->pixels));
+			printf("\t%s\t%c\t%s\n",
+					format_size(file->info->size),
 					file->info->has_alpha ? 'X' : '-', file->filename);
+		}
 
 		feh_action_run(file, opt.actions[0]);
 	}
@@ -74,6 +77,7 @@ void real_loadables_mode(int loadable)
 {
 	feh_file *file;
 	gib_list *l;
+	char ret = 0;
 
 	opt.quiet = 1;
 
@@ -88,6 +92,8 @@ void real_loadables_mode(int loadable)
 				puts(file->filename);
 				feh_action_run(file, opt.actions[0]);
 			}
+			else
+				ret = 1;
 			gib_imlib_free_image_and_decache(im);
 		} else {
 			/* Oh dear. */
@@ -95,7 +101,9 @@ void real_loadables_mode(int loadable)
 				puts(file->filename);
 				feh_action_run(file, opt.actions[0]);
 			}
+			else
+				ret = 1;
 		}
 	}
-	exit(0);
+	exit(ret);
 }
