@@ -58,14 +58,14 @@ void init_slideshow_mode(void)
 			last = NULL;
 		}
 		current_file = l;
-		s = slideshow_create_name(file);
+		s = slideshow_create_name(file, NULL);
 		if ((w = winwidget_create_from_file(l, s, WIN_TYPE_SLIDESHOW)) != NULL) {
 			free(s);
 			success = 1;
 			winwidget_show(w);
 			if (opt.slideshow_delay > 0.0)
 				feh_add_timer(cb_slide_timer, w, opt.slideshow_delay, "SLIDE_CHANGE");
-			else if (opt.reload > 0)
+			if (opt.reload > 0)
 				feh_add_unique_timer(cb_reload_timer, w, opt.reload);
 			break;
 		} else {
@@ -76,8 +76,6 @@ void init_slideshow_mode(void)
 	if (!success)
 		show_mini_usage();
 
-	setup_signal_handlers();
-
 	return;
 }
 
@@ -135,7 +133,7 @@ void cb_reload_timer(void *data)
 	/* reset window name in case of current file order,
 	 * filename, or filelist_length has changed.
 	 */
-	current_filename = slideshow_create_name(FEH_FILE(current_file->data));
+	current_filename = slideshow_create_name(FEH_FILE(current_file->data), w);
 	winwidget_rename(w, current_filename);
 	free(current_filename);
 
@@ -151,6 +149,16 @@ void feh_reload_image(winwidget w, int resize, int force_new)
 	Imlib_Image tmp;
 	int old_w, old_h;
 
+	unsigned char tmode =0;
+	int tim_x =0;
+	int tim_y =0;
+	double tzoom =0;
+
+	tmode = w->mode;
+	tim_x = w->im_x;
+	tim_y = w->im_y;
+	tzoom = w->zoom;
+
 	if (!w->file) {
 		im_weprintf(w, "couldn't reload, this image has no file associated with it.");
 		winwidget_render_image(w, 0, 0);
@@ -218,7 +226,16 @@ void feh_reload_image(winwidget w, int resize, int force_new)
 		w->im_w = gib_imlib_image_get_width(w->im);
 		w->im_h = gib_imlib_image_get_height(w->im);
 	}
-	winwidget_render_image(w, resize, 0);
+	if (opt.keep_zoom_vp) {
+		/* put back in: */
+		w->mode = tmode;
+		w->im_x = tim_x;
+		w->im_y = tim_y;
+		w->zoom = tzoom;
+		winwidget_render_image(w, 0, 0);
+	} else {
+		winwidget_render_image(w, resize, 0);
+	}
 
 	winwidget_rename(w, title);
 	free(title);
@@ -238,6 +255,11 @@ void slideshow_change_image(winwidget winwid, int change, int render)
 	int our_filelist_len = filelist_len;
 	char *s;
 
+	unsigned char tmode =0;
+	int tim_x =0;
+	int tim_y =0;
+	double tzoom =0;
+
 	/* Without this, clicking a one-image slideshow reloads it. Not very *
 	   intelligent behaviour :-) */
 	if (filelist_len < 2 && opt.cycle_once == 0)
@@ -266,8 +288,11 @@ void slideshow_change_image(winwidget winwid, int change, int render)
 			current_file = feh_list_jump(filelist, current_file, BACK, 1);
 			break;
 		case SLIDE_RAND:
-			current_file = feh_list_jump(filelist, current_file, FORWARD, rand() % filelist_len);
-			change = SLIDE_NEXT;
+			if (filelist_len > 1) {
+				current_file = feh_list_jump(filelist, current_file, FORWARD,
+					(rand() % (filelist_len - 1)) + 1);
+				change = SLIDE_NEXT;
+			}
 			break;
 		case SLIDE_JUMP_FWD:
 			if (filelist_len < 5)
@@ -307,6 +332,14 @@ void slideshow_change_image(winwidget winwid, int change, int render)
 			last = NULL;
 		}
 
+		if (opt.keep_zoom_vp) {
+		/* pre loadimage - record settings */
+			tmode = winwid->mode;
+			tim_x = winwid->im_x;
+			tim_y = winwid->im_y;
+			tzoom = winwid->zoom;
+		}
+
 		if ((winwidget_loadimage(winwid, FEH_FILE(current_file->data)))
 		    != 0) {
 			winwid->mode = MODE_NORMAL;
@@ -317,10 +350,22 @@ void slideshow_change_image(winwidget winwid, int change, int render)
 			winwidget_reset_image(winwid);
 			winwid->im_w = gib_imlib_image_get_width(winwid->im);
 			winwid->im_h = gib_imlib_image_get_height(winwid->im);
-			if (render)
-				winwidget_render_image(winwid, 1, 0);
+			if (opt.keep_zoom_vp) {
+				/* put back in: */
+				winwid->mode = tmode;
+				winwid->im_x = tim_x;
+				winwid->im_y = tim_y;
+				winwid->zoom = tzoom;
+			}
+			if (render) {
+				if (opt.keep_zoom_vp) {
+					winwidget_render_image(winwid, 0, 0);
+				} else {
+					winwidget_render_image(winwid, 1, 0);
+				}
+			}
 
-			s = slideshow_create_name(FEH_FILE(current_file->data));
+			s = slideshow_create_name(FEH_FILE(current_file->data), winwid);
 			winwidget_rename(winwid, s);
 			free(s);
 
@@ -350,7 +395,7 @@ void slideshow_pause_toggle(winwidget w)
 	winwidget_rename(w, NULL);
 }
 
-char *slideshow_create_name(feh_file * file)
+char *slideshow_create_name(feh_file * file, winwidget winwid)
 {
 	char *s = NULL;
 	int len = 0;
@@ -361,7 +406,7 @@ char *slideshow_create_name(feh_file * file)
 		snprintf(s, len, PACKAGE " [%d of %d] - %s",
 			gib_list_num(filelist, current_file) + 1, gib_list_length(filelist), file->filename);
 	} else {
-		s = estrdup(feh_printf(opt.title, file));
+		s = estrdup(feh_printf(opt.title, file, winwid));
 	}
 
 	return(s);
@@ -372,7 +417,7 @@ void feh_action_run(feh_file * file, char *action)
 	if (action) {
 		char *sys;
 		D(("Running action %s\n", action));
-		sys = feh_printf(action, file);
+		sys = feh_printf(action, file, NULL);
 
 		if (opt.verbose && !opt.list && !opt.customlist)
 			fprintf(stderr, "Running action -->%s<--\n", sys);
@@ -404,13 +449,28 @@ char *shell_escape(char *input)
 	return ret;
 }
 
-char *feh_printf(char *str, feh_file * file)
+char *format_size(int size)
+{
+	static char ret[5];
+	char units[] = {' ', 'k', 'M', 'G', 'T'};
+	unsigned char postfix = 0;
+	while (size >= 1000) {
+		size /= 1000;
+		postfix++;
+	}
+	snprintf(ret, 5, "%3d%c", size, units[postfix]);
+	return ret;
+}
+
+char *feh_printf(char *str, feh_file * file, winwidget winwid)
 {
 	char *c;
 	char buf[20];
 	static char ret[4096];
+	char *filelist_tmppath;
 
 	ret[0] = '\0';
+	filelist_tmppath = NULL;
 
 	for (c = str; *c != '\0'; c++) {
 		if ((*c == '%') && (*(c+1) != '\0')) {
@@ -418,84 +478,116 @@ char *feh_printf(char *str, feh_file * file)
 			switch (*c) {
 			case 'f':
 				if (file)
-					strcat(ret, file->filename);
+					strncat(ret, file->filename, sizeof(ret) - strlen(ret) - 1);
 				break;
 			case 'F':
 				if (file)
-					strcat(ret, shell_escape(file->filename));
-				break;
-			case 'n':
-				if (file)
-					strcat(ret, file->name);
-				break;
-			case 'N':
-				if (file)
-					strcat(ret, shell_escape(file->name));
-				break;
-			case 'w':
-				if (file && (file->info || !feh_file_info_load(file, NULL))) {
-					snprintf(buf, sizeof(buf), "%d", file->info->width);
-					strcat(ret, buf);
-				}
+					strncat(ret, shell_escape(file->filename), sizeof(ret) - strlen(ret) - 1);
 				break;
 			case 'h':
 				if (file && (file->info || !feh_file_info_load(file, NULL))) {
 					snprintf(buf, sizeof(buf), "%d", file->info->height);
-					strcat(ret, buf);
+					strncat(ret, buf, sizeof(ret) - strlen(ret) - 1);
 				}
 				break;
-			case 's':
-				if (file && (file->info || !feh_file_info_load(file, NULL))) {
-					snprintf(buf, sizeof(buf), "%d", file->info->size);
-					strcat(ret, buf);
+			case 'l':
+				snprintf(buf, sizeof(buf), "%d", gib_list_length(filelist));
+				strncat(ret, buf, sizeof(ret) - strlen(ret) - 1);
+				break;
+			case 'L':
+				if (filelist_tmppath != NULL) {
+					strncat(ret, filelist_tmppath, sizeof(ret) - strlen(ret) - 1);
+				} else {
+					filelist_tmppath = feh_unique_filename("/tmp/","filelist");
+					feh_write_filelist(filelist, filelist_tmppath);
+					strncat(ret, filelist_tmppath, sizeof(ret) - strlen(ret) - 1);
 				}
 				break;
-			case 'S':
-				if (file && (file->info || !feh_file_info_load(file, NULL))) {
-					snprintf(buf, sizeof(buf),
-							"%.2fkB", ((double)file->info->size / 1000));
-					strcat(ret, buf);
+			case 'm':
+				strncat(ret, mode, sizeof(ret) - strlen(ret) - 1);
+				break;
+			case 'n':
+				if (file)
+					strncat(ret, file->name, sizeof(ret) - strlen(ret) - 1);
+				break;
+			case 'N':
+				if (file)
+					strncat(ret, shell_escape(file->name), sizeof(ret) - strlen(ret) - 1);
+				break;
+			case 'o':
+				if (winwid) {
+					snprintf(buf, sizeof(buf), "%d,%d", winwid->im_x,
+						winwid->im_y);
+					strncat(ret, buf, sizeof(ret) - strlen(ret) - 1);
 				}
 				break;
 			case 'p':
 				if (file && (file->info || !feh_file_info_load(file, NULL))) {
 					snprintf(buf, sizeof(buf), "%d", file->info->pixels);
-					strcat(ret, buf);
+					strncat(ret, buf, sizeof(ret) - strlen(ret) - 1);
+				}
+				break;
+			case 'P':
+				if (file && (file->info || !feh_file_info_load(file, NULL))) {
+					strncat(ret, format_size(file->info->pixels), sizeof(ret) - strlen(ret) - 1);
+				}
+				break;
+			case 'r':
+				if (winwid) {
+					snprintf(buf, sizeof(buf), "%.1f", winwid->im_angle);
+					strncat(ret, buf, sizeof(ret) - strlen(ret) - 1);
+				}
+				break;
+			case 's':
+				if (file && (file->info || !feh_file_info_load(file, NULL))) {
+					snprintf(buf, sizeof(buf), "%d", file->info->size);
+					strncat(ret, buf, sizeof(ret) - strlen(ret) - 1);
+				}
+				break;
+			case 'S':
+				if (file && (file->info || !feh_file_info_load(file, NULL))) {
+					strncat(ret, format_size(file->info->size), sizeof(ret) - strlen(ret) - 1);
 				}
 				break;
 			case 't':
 				if (file && (file->info || !feh_file_info_load(file, NULL))) {
-					strcat(ret, file->info->format);
+					strncat(ret, file->info->format, sizeof(ret) - strlen(ret) - 1);
 				}
 				break;
-			case 'P':
-				strcat(ret, PACKAGE);
-				break;
-			case 'v':
-				strcat(ret, VERSION);
-				break;
-			case 'm':
-				strcat(ret, mode);
-				break;
-			case 'l':
-				snprintf(buf, sizeof(buf), "%d", gib_list_length(filelist));
-				strcat(ret, buf);
-				break;
 			case 'u':
 				snprintf(buf, sizeof(buf), "%d",
 					 current_file != NULL ? gib_list_num(filelist, current_file)
 					 + 1 : 0);
-				strcat(ret, buf);
+				strncat(ret, buf, sizeof(ret) - strlen(ret) - 1);
+				break;
+			case 'v':
+				strncat(ret, VERSION, sizeof(ret) - strlen(ret) - 1);
+				break;
+			case 'V':
+				snprintf(buf, sizeof(buf), "%d", getpid());
+				strncat(ret, buf, sizeof(ret) - strlen(ret) - 1);
+				break;
+			case 'w':
+				if (file && (file->info || !feh_file_info_load(file, NULL))) {
+					snprintf(buf, sizeof(buf), "%d", file->info->width);
+					strncat(ret, buf, sizeof(ret) - strlen(ret) - 1);
+				}
+				break;
+			case 'z':
+				if (winwid) {
+					snprintf(buf, sizeof(buf), "%.2f", winwid->zoom);
+					strncat(ret, buf, sizeof(ret) - strlen(ret) - 1);
+				}
 				break;
 			case '%':
-				strcat(ret, "%");
+				strncat(ret, "%", sizeof(ret) - strlen(ret) - 1);
 				break;
 			default:
 				weprintf("Unrecognized format specifier %%%c", *c);
 				strncat(ret, c - 1, 2);
 				break;
 			}
-		} else if ((*c == '\\') && (*(c+1) != '\0')) {
+		} else if ((*c == '\\') && (*(c+1) != '\0') && ((strlen(ret) + 3) < sizeof(ret))) {
 			c++;
 			switch (*c) {
 			case 'n':
@@ -505,9 +597,11 @@ char *feh_printf(char *str, feh_file * file)
 				strncat(ret, c - 1, 2);
 				break;
 			}
-		} else
+		} else if ((strlen(ret) + 2) < sizeof(ret))
 			strncat(ret, c, 1);
 	}
+	if (filelist_tmppath != NULL)
+		free(filelist_tmppath);
 	return(ret);
 }
 
@@ -518,6 +612,14 @@ void feh_filelist_image_remove(winwidget winwid, char do_delete)
 		gib_list *doomed;
 
 		doomed = current_file;
+		/*
+		 * work around feh_list_jump exiting if cycle_once is enabled
+		 * and no further files are left (we need to delete first)
+		 */
+		if (opt.cycle_once && ! doomed->next && do_delete) {
+			feh_file_rm_and_free(filelist, doomed);
+			exit(0);
+		}
 		slideshow_change_image(winwid, SLIDE_NEXT, 0);
 		if (do_delete)
 			filelist = feh_file_rm_and_free(filelist, doomed);
@@ -528,7 +630,7 @@ void feh_filelist_image_remove(winwidget winwid, char do_delete)
 			winwidget_destroy(winwid);
 			return;
 		}
-		s = slideshow_create_name(FEH_FILE(winwid->file->data));
+		s = slideshow_create_name(FEH_FILE(winwid->file->data), winwid);
 		winwidget_rename(winwid, s);
 		free(s);
 		winwidget_render_image(winwid, 1, 0);
@@ -558,19 +660,13 @@ void slideshow_save_image(winwidget win)
 		tmpname = feh_unique_filename("", "noname.png");
 	}
 
-	if (!opt.quiet)
+	if (opt.verbose)
 		printf("saving image to filename '%s'\n", tmpname);
 
-	/* XXX gib_imlib_save_image_with_error_return breaks with *.XXX and
-	 * similar because it tries to set the image format, which only works
-	 * with .xxx .
-	 * So we leave that part out.
-	 */
-	imlib_context_set_image(win->im);
-	imlib_save_image_with_error_return(tmpname, &err);
+	ungib_imlib_save_image_with_error_return(win->im, tmpname, &err);
 
 	if (err)
-		im_weprintf(win, "Can't save image %s:", tmpname);
+		feh_imlib_print_load_error(tmpname, win, err);
 
 	free(tmpname);
 	return;
