@@ -32,6 +32,7 @@ CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 #include "md5.h"
 #include "feh_png.h"
 #include "index.h"
+#include "signals.h"
 
 static gib_list *thumbnails = NULL;
 
@@ -54,6 +55,7 @@ void init_thumbnail_mode(void)
 	   int max_column_w = 0;
 	 */
 
+	Imlib_Load_Error err;
 	Imlib_Image im_temp;
 	int ww = 0, hh = 0, www, hhh, xxx, yyy;
 	int orig_w, orig_h;
@@ -168,7 +170,7 @@ void init_thumbnail_mode(void)
 	if (!opt.title)
 		s = estrdup(PACKAGE " [thumbnail mode]");
 	else
-		s = estrdup(feh_printf(opt.title, NULL));
+		s = estrdup(feh_printf(opt.title, NULL, NULL));
 
 	if (opt.display) {
 		winwid = winwidget_create_from_image(td.im_main, s, WIN_TYPE_THUMBNAIL);
@@ -380,8 +382,11 @@ void init_thumbnail_mode(void)
 			snprintf(output_buf, 1024, "%s/%s", opt.output_dir, opt.output_file);
 		else
 			strncpy(output_buf, opt.output_file, 1024);
-		gib_imlib_save_image(td.im_main, output_buf);
-		if (opt.verbose) {
+		ungib_imlib_save_image_with_error_return(td.im_main, output_buf, &err);
+		if (err) {
+			feh_imlib_print_load_error(output_buf, td.im_main, err);
+		}
+		else if (opt.verbose) {
 			int tw, th;
 
 			tw = gib_imlib_image_get_width(td.im_main);
@@ -395,6 +400,16 @@ void init_thumbnail_mode(void)
 
 	if (!opt.display)
 		gib_imlib_free_image_and_decache(td.im_main);
+	else if (opt.start_list_at) {
+		for (l = thumbnails; l; l = l->next) {
+			if (!strcmp(opt.start_list_at, FEH_THUMB(l->data)->file->filename)) {
+				opt.start_list_at = NULL;
+				feh_thumbnail_select(winwid, FEH_THUMB(l->data));
+				break;
+			}
+		}
+	}
+
 
 	free(s);
 	return;
@@ -595,6 +610,9 @@ char *feh_thumbnail_get_name_uri(char *name)
 		/* FIXME: add support for ~, need to investigate if it's expanded
 		   somewhere else before adding (unecessary) code */
 		if (name[0] != '/') {
+			/* work around /some/path/./image.ext */
+			if ((strncmp(name, "./", 2)) == 0)
+				name += 2;
 			cwd = getcwd(NULL, 0);
 			uri = estrjoin("/", "file:/", cwd, name, NULL);
 			free(cwd);
@@ -716,7 +734,7 @@ void feh_thumbnail_show_fullsize(feh_file *thumbfile)
 	if (!opt.thumb_title)
 		s = thumbfile->name;
 	else
-		s = feh_printf(opt.thumb_title, thumbfile);
+		s = feh_printf(opt.thumb_title, thumbfile, NULL);
 	
 	thumbwin = winwidget_get_first_window_of_type(WIN_TYPE_THUMBNAIL_VIEWER);
 	if (!thumbwin) {
@@ -768,45 +786,56 @@ void feh_thumbnail_select(winwidget winwid, feh_thumbnail *thumbnail)
 
 void feh_thumbnail_select_next(winwidget winwid, int jump)
 {
-	gib_list *l, *tmp;
-	int i;
+	gib_list *l;
+	feh_thumbnail *thumb;
+	int len = 0, cur = 0, target = 0;
 
-	for (l = thumbnails; l && l->next; l = l->next) {
-		tmp = l;
-		for (i = jump; (i > 0) && tmp->next; i--)
-			tmp = tmp->next;
-		if (tmp->data == td.selected)
-			break;
+	for (l = thumbnails; l; l = l->next) {
+		thumb = FEH_THUMB(l->data);
+		if (thumb == td.selected)
+			cur = len;
+		len++;
 	}
 
-	feh_thumbnail_select(winwid, FEH_THUMB(l->data));
+	target = (cur + len - jump) % len;
+
+	for (l = thumbnails; l; l = l->next) {
+		if (target-- == 0) {
+			feh_thumbnail_select(winwid, FEH_THUMB(l->data));
+		}
+	}
 }
 
 void feh_thumbnail_select_prev(winwidget winwid, int jump)
 {
 	gib_list *l;
 	feh_thumbnail *thumb;
-	int i;
+	int len = 0, cur = 0, target = 0;
 
 	for (l = thumbnails; l; l = l->next) {
 		thumb = FEH_THUMB(l->data);
-		if ((thumb == td.selected) && l->next) {
-			for (i = jump; (i > 0) && l->next; i--)
-				l = l->next;
+		if (thumb == td.selected)
+			cur = len;
+		len++;
+	}
+
+	target = (cur + jump) % len;
+
+	for (l = thumbnails; l; l = l->next) {
+		if (target-- == 0) {
 			feh_thumbnail_select(winwid, FEH_THUMB(l->data));
-			return;
+			break;
 		}
 	}
-	feh_thumbnail_select(winwid, FEH_THUMB(thumbnails->data));
 }
 
-inline void feh_thumbnail_show_selected()
+void feh_thumbnail_show_selected()
 {
 	if (td.selected && td.selected->file)
 		feh_thumbnail_show_fullsize(td.selected->file);
 }
 
-inline feh_file* feh_thumbnail_get_selected_file()
+feh_file* feh_thumbnail_get_selected_file()
 {
 	if (td.selected)
 		return td.selected->file;
