@@ -27,16 +27,13 @@ CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 #include "debug.h"
 #include "options.h"
 
-static char *feh_user_name = NULL;
-static char *feh_tmp_dir = NULL;
-
 /* eprintf: print error message and exit */
 void eprintf(char *fmt, ...)
 {
 	va_list args;
 
 	fflush(stdout);
-	fprintf(stderr, "%s ERROR: ", PACKAGE);
+	fputs(PACKAGE " ERROR: ", stderr);
 
 	va_start(args, fmt);
 	vfprintf(stderr, fmt, args);
@@ -44,7 +41,7 @@ void eprintf(char *fmt, ...)
 
 	if (fmt[0] != '\0' && fmt[strlen(fmt) - 1] == ':')
 		fprintf(stderr, " %s", strerror(errno));
-	fprintf(stderr, "\n");
+	fputs("\n", stderr);
 	exit(2);
 }
 
@@ -54,7 +51,7 @@ void weprintf(char *fmt, ...)
 	va_list args;
 
 	fflush(stdout);
-	fprintf(stderr, "%s WARNING: ", PACKAGE);
+	fputs(PACKAGE " WARNING: ", stderr);
 
 	va_start(args, fmt);
 	vfprintf(stderr, fmt, args);
@@ -62,7 +59,7 @@ void weprintf(char *fmt, ...)
 
 	if (fmt[0] != '\0' && fmt[strlen(fmt) - 1] == ':')
 		fprintf(stderr, " %s", strerror(errno));
-	fprintf(stderr, "\n");
+	fputs("\n", stderr);
 }
 
 /* estrdup: duplicate a string, report if error */
@@ -146,18 +143,6 @@ char *estrjoin(const char *separator, ...)
 	return string;
 }
 
-char *stroflen(char c, int l)
-{
-	static char buf[1024];
-	int i = 0;
-
-	buf[0] = '\0';
-	while (l--)
-		buf[i++] = c;
-	buf[i] = '\0';
-	return buf;
-}
-
 /* free the result please */
 char *feh_unique_filename(char *path, char *basename)
 {
@@ -205,32 +190,3 @@ char *ereadfile(char *path)
 
 	return estrdup(buffer);
 }
-
-char *feh_get_tmp_dir(void)
-{
-	char *tmp;
-	if (feh_tmp_dir)
-		return feh_tmp_dir;
-	tmp = getenv("TMPDIR");
-	if (!tmp)
-		tmp = getenv("TMP");
-	if (!tmp)
-		tmp = getenv("TEMP");
-	if (!tmp)
-		tmp = "/tmp";
-	feh_tmp_dir = estrdup(tmp);
-	return feh_tmp_dir;
-}
-
-char *feh_get_user_name(void)
-{
-	struct passwd *pw = NULL;
-
-	if (feh_user_name)
-		return feh_user_name;
-	setpwent();
-	pw = getpwuid(getuid());
-	endpwent();
-	feh_user_name = estrdup(pw->pw_name);
-	return feh_user_name;
-}
