@@ -190,6 +190,9 @@ static void feh_wm_set_bg_maxed(Pixmap pmap, Imlib_Image im, int use_filelist,
 void feh_wm_set_bg(char *fil, Imlib_Image im, int centered, int scaled,
 		int filled, int desktop, int use_filelist)
 {
+	XGCValues gcvalues;
+	XGCValues gcval;
+	GC gc;
 	char bgname[20];
 	int num = (int) rand();
 	char bgfil[4096];
@@ -257,13 +260,13 @@ void feh_wm_set_bg(char *fil, Imlib_Image im, int centered, int scaled,
 		char *home;
 		char filbuf[4096];
 		char fehbg_xinerama[] = "--no-xinerama";
+		char *bgfill = NULL;
+		bgfill = opt.image_bg == IMAGE_BG_WHITE ?  "--image-bg white" : "--image-bg black" ;
 
 		/* local display to set closedownmode on */
 		Display *disp2;
 		Window root2;
 		int depth2;
-		XGCValues gcvalues;
-		GC gc;
 		int in, out, w, h;
 
 		if (opt.xinerama)
@@ -320,13 +323,14 @@ void feh_wm_set_bg(char *fil, Imlib_Image im, int centered, int scaled,
 					0, 0, scr->width, scr->height);
 			fehbg = estrjoin(" ", "feh", fehbg_xinerama, "--bg-scale", filbuf, NULL);
 		} else if (centered) {
-			XGCValues gcval;
-			GC gc;
 
 			D(("centering\n"));
 
 			pmap_d1 = XCreatePixmap(disp, root, scr->width, scr->height, depth);
-			gcval.foreground = BlackPixel(disp, DefaultScreen(disp));
+			if (opt.image_bg == IMAGE_BG_WHITE)
+				gcval.foreground = WhitePixel(disp, DefaultScreen(disp));
+			else
+				gcval.foreground = BlackPixel(disp, DefaultScreen(disp));
 			gc = XCreateGC(disp, root, GCForeground, &gcval);
 			XFillRectangle(disp, pmap_d1, gc, 0, 0, scr->width, scr->height);
 
@@ -343,7 +347,7 @@ void feh_wm_set_bg(char *fil, Imlib_Image im, int centered, int scaled,
 
 			XFreeGC(disp, gc);
 
-			fehbg = estrjoin(" ", "feh", fehbg_xinerama, "--bg-center", filbuf, NULL);
+			fehbg = estrjoin(" ", "feh", fehbg_xinerama, bgfill, "--bg-center", filbuf, NULL);
 
 		} else if (filled == 1) {
 
@@ -363,10 +367,12 @@ void feh_wm_set_bg(char *fil, Imlib_Image im, int centered, int scaled,
 			fehbg = estrjoin(" ", "feh", fehbg_xinerama, "--bg-fill", filbuf, NULL);
 
 		} else if (filled == 2) {
-			XGCValues gcval;
 
 			pmap_d1 = XCreatePixmap(disp, root, scr->width, scr->height, depth);
-			gcval.foreground = BlackPixel(disp, DefaultScreen(disp));
+			if (opt.image_bg == IMAGE_BG_WHITE)
+				gcval.foreground = WhitePixel(disp, DefaultScreen(disp));
+			else
+				gcval.foreground = BlackPixel(disp, DefaultScreen(disp));
 			gc = XCreateGC(disp, root, GCForeground, &gcval);
 			XFillRectangle(disp, pmap_d1, gc, 0, 0, scr->width, scr->height);
 
@@ -383,7 +389,7 @@ void feh_wm_set_bg(char *fil, Imlib_Image im, int centered, int scaled,
 
 			XFreeGC(disp, gc);
 
-			fehbg = estrjoin(" ", "feh", fehbg_xinerama, "--bg-max", filbuf, NULL);
+			fehbg = estrjoin(" ", "feh", fehbg_xinerama, bgfill, "--bg-max", filbuf, NULL);
 
 		} else {
 			if (use_filelist)
