@@ -42,6 +42,7 @@ static winwidget winwidget_allocate(void)
 	winwidget ret = NULL;
 
 	ret = emalloc(sizeof(_winwidget));
+	memset(ret, 0, sizeof(_winwidget));
 
 	ret->win = 0;
 	ret->w = 0;
@@ -401,6 +402,7 @@ void winwidget_render_image(winwidget winwid, int resize, int force_alias)
 
 	if (!winwid->full_screen && opt.scale_down && ((winwid->w < winwid->im_w)
 						       || (winwid->h < winwid->im_h)) &&
+							  (winwid->type != WIN_TYPE_THUMBNAIL) &&
 							  (winwid->old_zoom == 1.0)) {
 		D(("scaling down image %dx%d\n", winwid->w, winwid->h));
 
@@ -410,19 +412,20 @@ void winwidget_render_image(winwidget winwid, int resize, int force_alias)
 		D(("after scaling down image %dx%d\n", winwid->w, winwid->h));
 	}
 
-	if (!winwid->full_screen && ((gib_imlib_image_has_alpha(winwid->im)) || (opt.geom_flags)
+	if (!winwid->full_screen && ((gib_imlib_image_has_alpha(winwid->im))
+				     || (opt.geom_flags & (WidthValue | HeightValue))
 				     || (winwid->im_x || winwid->im_y) || (winwid->zoom != 1.0)
 				     || (winwid->w > winwid->im_w || winwid->h > winwid->im_h)
 				     || (winwid->has_rotated)))
 		feh_draw_checks(winwid);
 
 	if (!winwid->full_screen && opt.zoom_mode
-				     && (winwid->zoom == 1.0) && !opt.geom_flags
+				     && (winwid->zoom == 1.0) && ! (opt.geom_flags & (WidthValue | HeightValue))
 				     && (winwid->w > winwid->im_w) && (winwid->h > winwid->im_h))
 		feh_calc_needed_zoom(&(winwid->zoom), winwid->im_w, winwid->im_h, winwid->w, winwid->h);
 
 
-	if (resize && (winwid->full_screen || opt.geom_flags)) {
+	if (resize && (winwid->full_screen || (opt.geom_flags & (WidthValue | HeightValue)))) {
 		int smaller;	/* Is the image smaller than screen? */
 		int max_w = 0, max_h = 0;
 
@@ -474,7 +477,7 @@ void winwidget_render_image(winwidget winwid, int resize, int force_alias)
 				double old_zoom = winwid->zoom;
 
 				winwid->zoom = 0.01 * opt.default_zoom;
-				if (winwid->zoom != 1.0) {
+				if (opt.default_zoom != 100) {
 					if ((winwid->im_h * winwid->zoom) > max_h)
 						winwid->zoom = old_zoom;
 					else if ((winwid->im_w * winwid->zoom) > max_w)
@@ -510,7 +513,8 @@ void winwidget_render_image(winwidget winwid, int resize, int force_alias)
 			winwid->im_y = (int) (max_h - (winwid->im_h * winwid->zoom)) >> 1;
 		}
 	}
-	else if (need_center && !winwid->full_screen && opt.scale_down) {
+	else if (need_center && !winwid->full_screen && opt.scale_down
+			&& (winwid->type != WIN_TYPE_THUMBNAIL)) {
 		winwid->im_x = (int) (winwid->w - (winwid->im_w * winwid->zoom)) >> 1;
 		winwid->im_y = (int) (winwid->h - (winwid->im_h * winwid->zoom)) >> 1;
 	}
@@ -574,6 +578,10 @@ void winwidget_render_image(winwidget winwid, int resize, int force_alias)
 			winwidget_update_caption(winwid);
 		if (opt.draw_filename)
 			feh_draw_filename(winwid);
+#ifdef HAVE_LIBEXIF
+		if (opt.draw_exif)
+			feh_draw_exif(winwid);
+#endif
 		if (opt.draw_actions)
 			feh_draw_actions(winwid);
 		if (opt.draw_info && opt.info_cmd)
@@ -772,23 +780,28 @@ void winwidget_move(winwidget winwid, int x, int y)
 
 void winwidget_resize(winwidget winwid, int w, int h)
 {
-	Window ignored_window;
 	XWindowAttributes attributes;
-	int tc_x, tc_y;
+	int tc_x, tc_y, px, py;
 	int scr_width = scr->width;
 	int scr_height = scr->height;
 
+	/* discarded */
+	Window dw;
+	int di, i;
+	unsigned int du;
+
 	XGetWindowAttributes(disp, winwid->win, &attributes);
 
 #ifdef HAVE_LIBXINERAMA
 	if (opt.xinerama && xinerama_screens) {
-		int i;
 		xinerama_screen = 0;
+		XQueryPointer(disp, root, &dw, &dw, &px, &py, &di, &di, &du);
 		for (i = 0; i < num_xinerama_screens; i++) {
-			if (XY_IN_RECT(attributes.x, attributes.y,
+			if (XY_IN_RECT(px, py,
 						xinerama_screens[i].x_org,
 						xinerama_screens[i].y_org,
-						xinerama_screens[i].width, xinerama_screens[i].height)) {
+						xinerama_screens[i].width,
+						xinerama_screens[i].height)) {
 				xinerama_screen = i;
 				break;
 			}
@@ -806,7 +819,7 @@ void winwidget_resize(winwidget winwid, int w, int h)
 	D(("   x %d y %d w %d h %d\n", attributes.x, attributes.y, winwid->w,
 		winwid->h));
 
-	if (opt.geom_flags) {
+	if (opt.geom_flags & (WidthValue | HeightValue)) {
 		winwid->had_resize = 1;
 		return;
 	}
@@ -820,7 +833,7 @@ void winwidget_resize(winwidget winwid, int w, int h)
 			XTranslateCoordinates(disp, winwid->win, attributes.root,
 						-attributes.border_width -
 						attributes.x,
-						-attributes.border_width - attributes.y, &tc_x, &tc_y, &ignored_window);
+						-attributes.border_width - attributes.y, &tc_x, &tc_y, &dw);
 			winwid->x = tc_x;
 			winwid->y = tc_y;
 			XMoveResizeWindow(disp, winwid->win, tc_x, tc_y, winwid->w, winwid->h);
